/* Module CSOUND.C
   This file contains the hadnling of sound within the barcode sub-system. */

#include <allegro.h>
#include <string.h>

#include <cgui.h>
#include <cgui/mem.h>
#include <ccom.h>

#include "cbar.h"
#include "unit.h"
#include "bcs.h"
#include "csound.h"
#include "csound.ht"
#include "labldata.h"

#ifdef DJGPP
#include <pc.h>
#else
#define nosound()
#define sound(x)
#endif

typedef struct t_sound {
   int freq;                    /* The frequence of the ok-sound (if internal
                                   PC-speaker) */
   int duration;                /* The duration of the ok->sound (if internal
                                   PC-speaker) */
   int volume;                  /* The volume to play the sample with */
   struct SAMPLE *spl;          /* The loaded sound-sample */
   char *fn;                    /* The filename if external sound file is
                                   used */
   struct t_usetting *us;
} t_sound;

#define SoundDbid(str, state, name) sprintf(str, "%s%s", state, name)
#define SoundDbid(str, state, name) sprintf(str, "%s%s", state, name)
static char *sound_dbid_f = "frequence";
static char *sound_dbid_d = "duration";
static char *sound_dbid_v = "volume";
static char *sound_dbid_file = "file";

/* The sound-data
   --------------- */

static void BarSoundOff(void *data nouse)
{
   nosound();
}

extern void _CbarDestroySound(t_sound *s)
{
   if (s->spl)
      destroy_sample(s->spl);
   if (s->fn)
      Release(s->fn);
   Release(s);
}

extern t_sound *_CbarCreateSound(struct t_usetting *us)
{
   t_sound *s;

   s = GetMem0(t_sound, 1);
   s->us = us;
   return s;
}

extern void _CbarReplaceSoundData(t_sound *cs, t_sound *s)
{
   struct t_usetting *us;

   if (s->spl)
      destroy_sample(s->spl);
   if (s->fn)
      Release(s->fn);
   us = s->us;
   *s = *cs;
   s->us = us;
   cs->fn = NULL;
   cs->spl = NULL;
}

extern void _CbarDuplicateSound(t_sound *dup, t_sound *org)
{
   struct t_usetting *us;

   us = dup->us;
   *dup = *org;
   dup->us = us;
   dup->spl = NULL;
   if (org->fn)
      dup->fn = MkString(org->fn);
}

extern void _CbarLoadSound(t_sound *s)
{
   char *p, c;
   DATAFILE *dat;

   p = strchr(s->fn, '#');
   if (p) {
      p = get_filename(s->fn);
      c = p[-1];
      p[-1] = 0;
      dat = load_datafile_object(s->fn, p);
      p[-1] = c;
      if (dat && dat->type==DAT_SAMPLE) {
         s->spl = dat->dat;
         dat->dat = NULL;
      }
      unload_datafile_object(dat);
   } else
      s->spl = load_sample(s->fn);
}

extern void _CbarStoreSoundConfig(char *section, t_sound *s, char *readstate)
{
   char sarr[200];

   SoundDbid(sarr, readstate, sound_dbid_f);
   set_config_int(section, sarr, s->freq);
   SoundDbid(sarr, readstate, sound_dbid_d);
   set_config_int(section, sarr, s->duration);
   SoundDbid(sarr, readstate, sound_dbid_v);
   set_config_int(section, sarr, s->volume);
   if (s->fn) {
      SoundDbid(sarr, readstate, sound_dbid_file);
      set_config_string(section, sarr, s->fn);
   }
}

extern void _CbarLoadSoundConfig(char *section, t_sound *s, char *readstate,
                      char *def)
{
   char sarr[200];

   SoundDbid(sarr, readstate, sound_dbid_f);
   s->freq = get_config_int(section, sarr, 1000);
   SoundDbid(sarr, readstate, sound_dbid_d);
   s->duration = get_config_int(section, sarr, 200);
   SoundDbid(sarr, readstate, sound_dbid_v);
   s->volume = get_config_int(section, sarr, 255);
   if (s->volume > 255)
      s->volume = 255;
   SoundDbid(sarr, readstate, sound_dbid_file);
   s->fn = MkString(get_config_string(section, sarr, def));
}

extern int _CbarSetExternalSpeakers(int newstate)
{
   static int sound_installed = 0;
   int org_state;

   org_state = sound_installed;
   if (newstate) {
      if (!sound_installed) {
         if (install_sound(DIGI_AUTODETECT, MIDI_NONE, "") == 0) {
            sound_installed = 1;
            set_volume(255, -1);
         }
      }
   } else if (sound_installed) {
      remove_sound();
      sound_installed = 0;
   }
   return org_state;
}

extern void _CbarSoundPropExternal(t_sound *s, int vol, const char *fn)
{
   s->volume = vol;
   if (s->fn)
      Release(s->fn);
   s->fn = MkString(fn);
}

extern void _CbarSoundPropInternal(t_sound *s, int freq, int duration)
{
   s->freq = freq;
   s->duration = duration;
}

extern int _CbarSound(t_sound *s)
{
   int error = 0;

   if (s->us->intsound) {
      sound(s->freq);
      GenEvent(BarSoundOff, NULL, s->duration, 0);
   }
   if (s->us->leftsound || s->us->rightsound) {
      if (s->spl == NULL) {
         _CbarLoadSound(s);
         if (s->spl == NULL) {
            if (s->freq > 20 && s->duration > 50)
               s->spl = create_sample(8, 0, s->freq, s->duration);
         }
      }
      if (s->spl == NULL)
         return 0;
      if (s->spl && s->us->leftsound)
         error |= play_sample(s->spl, s->volume, 0, 1000, 0) < 0;
      if (s->spl && s->us->rightsound)
         error |= play_sample(s->spl, s->volume, 255, 1000, 0) < 0;
   }
   return !error;
}

/* The dialogue data
   ---------------*/

#define ID_VOL    0
#define ID_CHECK  1
#define ID_HZ     2
#define ID_DUR    3
#define ID_FIL    4
#define ID_BROW   5
#define ID_NRTOT  6

typedef struct t_dialsound {
   int id[ID_NRTOT];            /* id-keys to win-object for
                                   activating/deactivating */
   int idcont;
   t_sound *s;            /* pointer to the copy of a sound object */
} t_dialsound;

extern void _CbarDestroyDialSound(t_dialsound *ds)
{
   Release(ds);
}

extern t_dialsound *_CbarCreateDialSound(t_sound *s)
{
   t_dialsound *ds;

   ds = GetMem0(t_dialsound, 1);
   ds->s = s;
   return ds;
}

extern int _CbarCheckInternalSoundSettings(t_sound *s, int ok)
{
   char tmp[1000];
   const char*const*txt;
   int nr;

   if (s->freq < 20 || s->duration < 55 || s->freq > 20000 || s->duration > 5000) {
      txt = LoadCompiledTexts(cbar_labels, "csound", &nr);
      if (nr < SIZE_csound)
         return 0;
      sprintf(tmp, txt[BAD_VALUES_FREQ_DURATION_KEEP_SETDEFAULT],
           s->freq, s->duration);
      if (Req("", tmp) == 1) {
         if (ok) {
            s->freq = 1000;
            s->duration = 200;
         } else {
            s->freq = 100;
            s->duration = 500;
         }
      }
   }
   return 1;
}

extern int _CbarCheckExternalSoundSettings(t_sound *s, const char *def)
{
   char tmp[1000];
   const char*const*txt;
   int nr;

   txt = LoadCompiledTexts(cbar_labels, "csound", &nr);
   if (nr < SIZE_csound)
      return 0;

   if (s->fn == NULL  || *s->fn == 0) {
      Req("", txt[NO_SOUND_FILE_TO_SOUND_CARD_CLOSE]);
      s->fn = MkString(def);
   }
   if (!exists(s->fn)) {
      sprintf(tmp, txt[SPECIFIED_SOUND_FILE_NOT_FOUND_OR_NOT_LOADABLE_CLOSE], s->fn);
      Req("", tmp);
      return 0;
   }
   if (s->volume == 0) {
      Req("", txt[NO_VOLUME_TO_SOUND_CARD_CLOSE]);
      return 0;
   }
   if (s->volume > 255)
      s->volume = 255;
   return 1;
}

/* The dialogue
   ------------*/

static void ViewSoundObject(SAMPLE *spl)
{
   if (spl) {
      const char*const*txt;
      int nr;
      txt = LoadCompiledTexts(cbar_labels, "csound", &nr);
      if (nr < SIZE_csound)
         return;
      play_sample(spl, 255, 0, 1000, 0);
      play_sample(spl, 255, 255, 1000, 0);
      AddTag(DOWNLEFT, txt[SOUND_SAMPLE]);
   }
}

static void SoundViewerData(void *data nouse, void *viewdata)
{
   DATAFILE *datob = viewdata;
   ViewSoundObject(datob->dat);
}

static void SoundViewer(void *data nouse, void *viewdata)
{
   const char *fn = viewdata;
   ViewSoundObject(load_sample(fn));
}

extern void _CbarReactivateObjects(t_dialsound *ds)
{
   int i;

   if (ds->s->us->leftsound || ds->s->us->rightsound) {
      for (i = 0; i < ID_NRTOT; i++)
         Activate(ds->id[i]);
   } else if (ds->s->us->intsound) {
         Activate(ds->id[ID_CHECK]);
         Activate(ds->id[ID_HZ]);
         Activate(ds->id[ID_DUR]);
         DeActivate(ds->id[ID_VOL]);
         DeActivate(ds->id[ID_FIL]);
         DeActivate(ds->id[ID_BROW]);
   } else {
      for (i = 0; i < ID_NRTOT; i++)
         DeActivate(ds->id[i]);
   }
   Refresh(ds->idcont);
}

static void PlaySoundSample(void *data)
{
   t_dialsound *ds = data;
   char tmp[1000];

   if (!_CbarSound(ds->s)) {
      const char*const*txt;
      int nr;
      txt = LoadCompiledTexts(cbar_labels, "csound", &nr);
      if (nr < SIZE_csound)
         return;
      sprintf(tmp, txt[SPECIFIED_SOUND_FILE_NOT_FOUND_OR_NOT_LOADABLE_CLOSE], ds->s->fn);
      Req("", tmp);
   }
}

static void StartBrowsing(void *data)
{
   t_dialsound *ds = data;
   t_sound *s;
   const char *fn;
   const char*const*txt;
   int nr;
   txt = LoadCompiledTexts(cbar_labels, "csound", &nr);
   if (nr < SIZE_csound)
      return;

   s = ds->s;
   RegisterFileType("wav", NULL, ds, "", NULL, txt[SOUND_SAMPLE], SoundViewer);
   RegisterFileType("#SAMP", NULL, ds, "", NULL, txt[SOUND_SAMPLE], SoundViewerData);
   fn = FileSelect("*.wav;*.voc", "", FS_DIRECT_SELECT_BY_DOUBLE_CLICK|FS_DISABLE_EDIT_DAT|FS_SHOW_MENU|FS_NO_DRAG_DROP|FS_SHOW_DIR_TREE,
                   txt[LOAD_SOUND_FILE], txt[SELECT]);
   if (*fn) {
      if (s->fn)
         Release(s->fn);
      s->fn = MkString(fn);
      if (s->spl) {
         destroy_sample(s->spl);
         s->spl = NULL;
      }
   }
   Refresh(ds->id[ID_FIL]);
}

static void EnteredValue(void *data)
{
   t_dialsound *ds = data;

   if (ds->s->spl) {
      destroy_sample(ds->s->spl);
      ds->s->spl = NULL;
   }
}

extern void _CbarMkSoundContainer(const char *tag, t_sound *s, t_dialsound *ds)
{
   const char*const*txt;
   int nr;
   txt = LoadCompiledTexts(cbar_labels, "csound", &nr);
   if (nr < SIZE_csound)
      return;
   /* the copy must be cleared - otherwise the original sample will be
      destroyed when user makes a sound-test and then cancels dialogue. */
   s->spl = NULL;

   ds->s = s;
   ds->idcont = StartContainer(DOWNLEFT | FILLSPACE | HORIZONTAL, ADAPTIVE, tag, CT_BORDER);
   ds->id[ID_CHECK] = AddButton(DOWNLEFT, txt[SOUND_CHECK], PlaySoundSample, ds);
   ds->id[ID_HZ] = AddEditBox(DOWNLEFT, 40, txt[FREQUENCE], FINT, 5, &s->freq);
   AddHandler(ds->id[ID_HZ], EnteredValue, ds);
   ds->id[ID_DUR] = AddEditBox(RIGHT, 40, txt[DURATION], FINT, 5, &s->duration);
   AddHandler(ds->id[ID_DUR], EnteredValue, ds);
   ds->id[ID_FIL] = AddEditBox(DOWNLEFT, 300, txt[SOUND_FILE], FPTRSTR, 0, &s->fn);
   AddHandler(ds->id[ID_FIL], EnteredValue, ds);
//txt[Browse]
   ds->id[ID_BROW] = AddButton(RIGHT, "#browse;...", StartBrowsing, ds);
   ds->id[ID_VOL] = AddEditBox(DOWNLEFT, 40, txt[VOLUME], FINT, 5, &s->volume);
   _CbarReactivateObjects(ds);
   EndContainer();
}
