/* File ERRORLOG.C.
   Defines the error logging of barcodes */

#include <cgui.h>
#include <cgui/mem.h>
#include <time.h>
#include <allegro.h>
#include <string.h>

#include "cbar.h"
#include "errorlog.h"
#include "errorlog.ht"
#include "labldata.h"

typedef struct t_log {
   char *typestring;
   char *code;
   int reader;
   time_t ti;
   struct t_log *next;
} t_log;

static t_log *start, *end;
static int logging, nrlogs, idlist;
static const char *const*txt;

static void DestroyLog2(t_log *log)
{
   if (log) {
      DestroyLog2(log->next);
      if (log->code)
         Release(log->code);
      if (log->typestring)
         Release(log->typestring);
      Release(log);
   }
}

static void DestroyLog(void)
{
   DestroyLog2(start);
   start = end = NULL;
   nrlogs = 0;
}

static void RefreshLogWin(void *data)
{
   int *pid = data;

   Refresh(*pid);
}

static void CloseLogWin(void *data)
{
   int *pid = data;

   *pid = 0;
   idlist = 0;
   CloseWin(NULL);
}

static void *GetLink(void *listdata, void *current)
{
   struct t_log **head=listdata,*cur=current;
   if (cur==NULL)
      return *head;
   else
      return cur->next;
}

static int DrawLogRow(void *rowdata, char *s, int colnr)
{
   t_log *log = rowdata;
   struct tm tm;

   *s = 0;
   switch (colnr) {
   case READER_NR:
      sprintf(s, "%d ", log->reader);
      return COL_RIGHT_ALIGN;
   case ERROR_TYPE:
      if (log->typestring)
         strcpy(s, log->typestring);
      break;
   case CODE_DIGITS:
      if (log->code)
         strcpy(s, log->code);
      break;
   case TIME:
      tm = *localtime(&log->ti);
      sprintf(s, "%02d:%02d:%02d", tm.tm_hour, tm.tm_min, tm.tm_sec);
/*      sprintf(s, "%d-%02d-%02d %02d:%02d", tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday, tm.tm_hour, tm.tm_min); */
      return COL_RIGHT_ALIGN;
   }
   return 0;
}

/* Application interface: */

extern void CbarLogErrors(int mode)
{
   if (mode & CBAR_LOG_CLEAN)
      DestroyLog();
   if (mode & CBAR_LOG_OFF)
      logging = 0;
   else if (mode & CBAR_LOG_ON)
      logging = 1;
}

extern void CbarAddToErrorLog(int reader, const char *error, const char *code)
{
   t_log *log;

   if (!logging)
      return;
   log = GetMem0(t_log, 1);
   log->ti = time(NULL);
   log->reader = reader;
   if (error)
      log->typestring = MkString(error);
   if (code)
      log->code = MkString(code);
   if (end) {
      end->next = log;
      end = log;
   } else {
      end = start = log;
   }
   nrlogs++;
   if (idlist)
      BrowseToL(idlist, nrlogs-1);
}

extern void CbarShowErrorLog(void)
{
   static int idwin, widths[NR_LOG_COLUMNS]={70, 180, 75, 75};
   int nr;

   if (idwin)
      SetFocusOn(idwin);
   else {
      txt = LoadCompiledTexts(cbar_labels, "errorlog", &nr);
      if (nr == SIZE_errorlog) {
         idwin = MkDialogue(ADAPTIVE, txt[ERROR_LOG], W_FLOATING);
         AddButton(DOWNLEFT, txt[REFRESH_LOG], RefreshLogWin, &idlist);
         idlist = AddList(DOWNLEFT, &start, &nrlogs, 400, 0, NULL, NULL, 20);
         SetLinkedList(idlist, GetLink);
         SetListColumns(idlist, DrawLogRow, widths, NR_LOG_COLUMNS,
                     LIST_COLUMNS_ADJUSTABLE, (char**)txt + LOG_COLUMNS, NULL, NULL);
         AddButton(DOWNLEFT, txt[CLOSE_LOG], CloseLogWin, &idwin);
         DisplayWin();
      }
   }
}
