/* File READER.C
   contains the functions that actually handles the reading of barcodes. */

#include <ctype.h>
#include <allegro.h>
#include <cgui.h>
#include <cgui/mem.h>
#include <ccom.h>
#include <string.h>

#include "cbar.h"
#include "labldata.h"
#include "bcs.h"
#include "unit.h"
#include "reader.h"
#include "reader.ht"
#include "setting.h"

#define MAXDYNLEN 1000

typedef struct t_reader {
   void (*CallBack) (int, char *, void *);
                                /* Application function called after each successfull reading */
   void *data;                  /* App data */
   char *buffer;                /* Buffer to put digits into during reading */
   char *code;                  /* Temp buffer for a sucessfully read code. */
   int n;                       /* Pointer to current digit in the buffer q; */
   t_rsetting *rs;
   int source;
   int mode;
   int seqno;
   int error_count;
   int error_type;
} t_reader;

static char Chk_10_31(char *s, int len)
{
   char c, oddc = 0, evenc = 0;
   int i;

   for (i = 1; i < len; i += 2)
      oddc += (s[i] - '0');
   for (i = 0; i < len; i += 2)
      evenc += (s[i] - '0');
   c = 9 - ((evenc * (len - 1) + oddc) % 10);
   return '0' + c;
}

static void BarCodeHandler(t_reader *r, char nextchar, int inerror)
{
   int len, fixlen, reqlen, readchk, error = 0;

   reqlen = r->rs->len;
   readchk = r->rs->readchk;
   fixlen =  reqlen >= 0;
   if (fixlen) {
      len = reqlen;
      if (readchk)
         len++;
   } else {
      len = r->n;
   }

   if (inerror) {
      error = -1;
   } else if (nextchar == '\r') {
      if (r->n < len)
         error = LOG_ERROR_TYPES + LOG_ERR_FEW_DIGITS;
      else if (readchk) {
         if (r->buffer[r->n - 1] != Chk_10_31(r->buffer, r->n - 1))
            error = LOG_ERROR_TYPES + LOG_ERR_CHK;
      }
      if (!error) {
         /* a complete code has been encountered and checksum is ok */
         if (r->CallBack) {
            if (readchk)
               r->buffer[r->n - 1] = 0;   /* rub checksum */
            else
               r->buffer[r->n] = 0;
            r->CallBack(r->seqno, r->buffer, r->data);
            r->n = 0;
         }
      }
   } else {
      if (fixlen) {
         if (r->n >= len)
            error = LOG_ERROR_TYPES + LOG_ERR_MISSING_CR;
      } else {
         if (r->n >= MAXDYNLEN)
            error = LOG_ERROR_TYPES + LOG_ERR_MISSING_CR;
      }
      if (!error)
         r->buffer[r->n++] = nextchar;
   }
   if (error) {
      const char *const*txt;
      int nr;

      r->error_count++;
      txt = LoadCompiledTexts(cbar_labels, "reader", &nr);
      if (nr == SIZE_reader) {
         r->buffer[r->n] = 0;
         if (r->error_count > 20) {
            return;
         } else if (r->error_count == 20) {
            CbarAddToErrorLog(r->seqno, txt[LOG_ERR_OVERFLOW], r->buffer);
         } else if (inerror) {
            char s[1000];
            sprintf(s, "%x", inerror);
            CbarAddToErrorLog(r->seqno, s, r->buffer);
            if (r->error_type != inerror)
               r->error_count = 0;
            r->error_count++;
            r->error_type = inerror;
         } else
            CbarAddToErrorLog(r->seqno, txt[error], r->buffer);
      }
      CbarMakeSound(r->seqno, CBAR_SOUND_READERROR);
      r->n = 0;
   } else
      r->error_count = 0;
}

static void COMBarCode(int portno nouse, int readbyte, int error, void *data)
{
   t_reader *r=data;

   BarCodeHandler(r, readbyte, error);
}

static int KeyBoardBarCode(void *data, int scan, int ascii)
{
   if (isdigit(ascii) || (ascii == '\r' && scan == KEY_ENTER))
      BarCodeHandler(data, ascii, 0);
   else
      return 0;
   return 1;
}

extern int _CbarIsReading(t_reader *r)
{
   return r->CallBack != NULL;
}

extern t_reader *_CbarStartReading(int source, t_rsetting *rs,
      void (*CallBack) (int, char *, void *), void *data, int mode, int seqno)
{
   t_reader *r;
   int len;

   r = GetMem0(t_reader, 1);
   r->n = 0;
   r->seqno = seqno;
   r->rs = rs;
   r->source = source;
   if (r->buffer)
      Release(r->buffer);
   len = r->rs->len;
   if (len)
      r->buffer = GetMem(char, len + 2);
   else
      r->buffer = GetMem(char, MAXDYNLEN + 2);
   if (r->source == CBAR_READER_KB)
      InstallKBHandler(KeyBoardBarCode, r);
   else
      CcomStartScanning(source-1, 0, COMBarCode, r);
   r->CallBack = CallBack;
   r->data = data;
   r->mode = mode;
   return r;
}

extern void _CbarStopReading(t_reader *r)
{
   if (r==NULL)
      return;
   if (r->source == CBAR_READER_KB)
      UnInstallKBHandler(KeyBoardBarCode);
   else
      CcomStopScanning(r->source-1);
   if (r->buffer) {
      Release(r->buffer);
      r->buffer = NULL;
   }
   r->CallBack = NULL;
   r->data = NULL;
   Release(r);
}

extern void CbarAdjustBarPrint(char *result, const char *s)
{
   int curlen, len, i = 0, missing, writechk;

   curlen = strlen(s);
   len = get_config_int(_cbc_bar_section_name, _cbc_bar_dbid_nr_digits, 0);
   writechk = get_config_int(_cbc_bar_section_name, _cbc_bar_dbid_write_check, 1);
   if (!len)
      len = curlen;
   if (writechk)
      len++;
   if (len & 1)
      len++;
   if (writechk)
      missing = len - curlen - 1;
   else
      missing = len - curlen;
   for (i = 0; i < missing; i++)
      result[i] = '0';
   strcpy(result + i, s);
   if (writechk) {
      result[len - 1] = 0;
      result[len - 1] = Chk_10_31(result, len - 1);
      result[len] = 0;
   }
}
