/* Module UNIT.C
   This file contains the dialogue functions for adpting one barcode
   unit. It also provides the possibility to adjust the serial port
   configuration, in case of a bar code reader connected to the serial port. */

#include <allegro.h>

#include <cgui.h>
#include <cgui/mem.h>
#include <ccom.h>

#include "cbar.h"
#include "unit.h"
#include "unit.ht"
#include "bcs.h"
#include "csound.h"
#include "reader.h"
#include "labldata.h"

typedef struct t_unit {
   t_usetting setting;
   struct t_sound *ok;             /* The sound properties of the ok-sound */
   struct t_sound *error;          /* The sound properties of the error-sound */
   struct t_reader *r;
   struct t_rsetting *rs;
} t_unit;

#define UnitSection(str, i) sprintf(str, unit_section_name, i+1)
#define defok  "barok.wav"
#define deferr "barerror.wav"
static char *unit_section_name = "Bar-reader%d";
static char *unit_dbid_src = "Reader-connection";
static char *unit_dbid_intsound = "Internal-speker";
static char *unit_dbid_leftsound = "Left-speker";
static char *unit_dbid_rightsound = "Right-speker";
static char *unit_ok = "OK-";
static char *unit_error = "Error-";

static void DeActivateUnit(t_unit *bcu)
{
   if (bcu->r) {
      _CbarStopReading(bcu->r);
      bcu->r = NULL;
   }
   if (bcu->ok) {
      _CbarDestroySound(bcu->ok);
      bcu->ok = NULL;
   }
   if (bcu->error) {
      _CbarDestroySound(bcu->error);
      bcu->error = NULL;
   }
   _CbarUnRegisterUseSound(bcu->setting.leftsound, bcu->setting.rightsound);
}

extern void _CbarStoreUnitConfig(t_unit *bcu, int i, const char *sec nouse)
{
   char section[200];

   UnitSection(section, i);
   set_config_int(section, unit_dbid_src, bcu->setting.source);
   set_config_int(section, unit_dbid_intsound, bcu->setting.intsound);
   set_config_int(section, unit_dbid_leftsound, bcu->setting.leftsound);
   set_config_int(section, unit_dbid_rightsound, bcu->setting.rightsound);
   _CbarStoreSoundConfig(section, bcu->ok, unit_ok);
   _CbarStoreSoundConfig(section, bcu->error, unit_error);
}

extern void _CbarLoadUnitConfig(t_unit *bcu, int seqno, const char *sec nouse)
{
   char section[200];

   UnitSection(section, seqno);
   bcu->setting.source = get_config_int(section, unit_dbid_src, 0);
   bcu->setting.intsound = get_config_int(section, unit_dbid_intsound, 0);
   bcu->setting.leftsound = get_config_int(section, unit_dbid_leftsound, 0);
   bcu->setting.rightsound = get_config_int(section, unit_dbid_rightsound, 0);
   _CbarLoadSoundConfig(section, bcu->ok, unit_ok, defok);
   _CbarLoadSoundConfig(section, bcu->error, unit_error, deferr);
}

extern void _CbarActivateUnit(t_unit *bcu,
            void (*CallBack) (int, char *, void *), void *data, int mode, int seqnr)
{
   DeActivateUnit(bcu);
   if (CallBack) {
      bcu->ok = _CbarCreateSound(&bcu->setting);
      bcu->error = _CbarCreateSound(&bcu->setting);
      _CbarLoadUnitConfig(bcu, seqnr, "");
      _CbarRegisterUseSound(bcu->setting.leftsound, bcu->setting.rightsound);
      _CbarLoadSound(bcu->ok);
      _CbarLoadSound(bcu->error);
      bcu->r = _CbarStartReading(bcu->setting.source, bcu->rs, CallBack, data,
               mode, seqnr);
   }
}

extern void _CbarDestroyUnit(t_unit *bcu)
{
   DeActivateUnit(bcu);
   Release(bcu);
}

static void ReplaceUnitData(t_unit *cbcu, t_unit *bcu)
{
   bcu->setting = cbcu->setting;
   _CbarReplaceSoundData(cbcu->ok, bcu->ok);
   _CbarReplaceSoundData(cbcu->error, bcu->error);
}

/* Application interface (settings): */

extern int CbarRemoveUnit(int seqnr)
{
   t_unit *bcu;

   bcu = _CbarGetUnit(seqnr);
   if (bcu == NULL)
      return 0;
   _CbarRemoveUnit(bcu);
   _CbarDestroyUnit(bcu);
   return 1;
}

extern int CbarUnitConnection(int seqnr, int src)
{
   t_unit *bcu;

   bcu = _CbarGetUnit(seqnr);
   if (bcu == NULL)
      return 0;
   bcu->setting.source = src;
   return 1;
}

extern int CbarSoundExternal(int seqnr, int speaker, int v1, const char *f1,
                                int v2, const char *f2)
{
   t_unit *bcu;

   bcu = _CbarGetUnit(seqnr);
   if (bcu == NULL)
      return 0;
   if (speaker & CBAR_LEFT_SPEAKER)
      bcu->setting.leftsound = 1;
   if (speaker & CBAR_RIGHT_SPEAKER)
      bcu->setting.leftsound = 1;
   _CbarSoundPropExternal(bcu->ok, v1, f1);
   _CbarSoundPropExternal(bcu->error, v2, f2);
   return 1;
}

extern int CbarSoundInternal(int seqnr, int f1, int d1, int f2, int d2)
{
   t_unit *bcu;

   bcu = _CbarGetUnit(seqnr);
   if (bcu == NULL)
      return 0;
   _CbarSoundPropInternal(bcu->ok, f1, d1);
   _CbarSoundPropInternal(bcu->error, f2, d2);
   return 1;
}

extern int CbarMakeSound(int seqnr, int soundstatus)
{
   struct t_sound *s = NULL;
   t_unit *bcu;

   bcu = _CbarGetUnit(seqnr);
   if (bcu == NULL)
      return 0;
   if (soundstatus == CBAR_SOUND_READERROR)
      s = bcu->error;
   else if (soundstatus == CBAR_SOUND_READOK)
      s = bcu->ok;
   if (s)
      _CbarSound(s);
   return 1;
}

static t_unit *DuplicateUnit(t_unit *org)
{
   t_unit *bcu;

   bcu = _CbarCreateUnit(org->rs);
   bcu->setting = org->setting;
   _CbarDuplicateSound(bcu->ok, org->ok);
   _CbarDuplicateSound(bcu->error, org->error);
   return bcu;
}

extern t_unit *_CbarCreateUnit(struct t_rsetting *rs)
{
   t_unit *bcu;

   bcu = GetMem0(t_unit, 1);
   bcu->ok = _CbarCreateSound(&bcu->setting);
   bcu->error = _CbarCreateSound(&bcu->setting);
   bcu->rs = rs;
   return bcu;
}

/* The dialogue data
   -----------------*/

typedef struct t_dialunit {
   int portid;                  /* Id of push-button (for deact./activating) */
   t_unit *bcu;                 /* A pointer to the original reader object */
   t_unit *cbcu;             /* A copy of the original reader data possibly
                                   modified by the user */
   int dsource;                 /* Tells if reader source is to be edited */
   int dsound;                  /* Tells if reader sound is to be edited */
   struct t_dialsound *okwr;             /* A wrapper to the ok-sound object */
   struct t_dialsound *errorwr;          /* A wrapper to the error-sound object */
} t_dialunit;

extern void _CbarSetEditedData(t_dialunit *du)
{
   if (du->cbcu && du->cbcu->setting.source != CBAR_READER_NONE) {
      if (du->bcu)
         ReplaceUnitData(du->cbcu, du->bcu);
      else {
         _CbarInsertUnit(du->cbcu);
         du->cbcu = NULL;
      }
   }
}

extern int _CbarCheckUniqueSources(t_dialunit *dr1, t_dialunit *dr2)
{
   const char*const*txt;
   int nr;

   if (dr1->cbcu->setting.source == dr2->cbcu->setting.source) {
      txt = LoadCompiledTexts(cbar_labels, "unit", &nr);
      if (nr < SIZE_unit)
         return 0;
      if (dr1->cbcu->setting.source == CBAR_READER_KB)
         Req("", txt[ONLY_ONE_KEYBOARD_READER_CLOSE]);
      else
          Req("", txt[ONLY_ONE_READER_PER_PORT_CLOSE]);
      return 0;
   }
   return 1;
}

extern int _CbarCheckEditedUnit(t_dialunit *du)
{
   t_unit *cbcu;

   cbcu = du->cbcu;
   if (cbcu->setting.intsound) {
      if (!_CbarCheckInternalSoundSettings(cbcu->ok, 1))
         return 0;
      if (!_CbarCheckInternalSoundSettings(cbcu->error, 0))
         return 0;
   }
   if (cbcu->setting.leftsound || cbcu->setting.rightsound) {
      if (!_CbarCheckExternalSoundSettings(cbcu->ok, defok))
         return 0;
      if (!_CbarCheckExternalSoundSettings(cbcu->error, deferr))
         return 0;
   }
   return 1;
}

extern t_dialunit *_CbarCreateDialUnit(t_unit *bcu, int dsource, int dsound,
                  struct t_rsetting *rs)
{
   t_dialunit *du;
   const char*const*txt;
   int nr;
   txt = LoadCompiledTexts(cbar_labels, "unit", &nr);
   if (nr < SIZE_unit)
      return 0;

   du = GetMem0(t_dialunit, 1);
   if (bcu)
      du->cbcu = DuplicateUnit(bcu);
   else
      du->cbcu = _CbarCreateUnit(rs);
   du->bcu = bcu;
   du->dsource = dsource;
   du->dsound = dsound;
   if (dsound) {
      du->okwr = _CbarCreateDialSound(du->cbcu->ok);
      du->errorwr = _CbarCreateDialSound(du->cbcu->error);
   }
   return du;
}

extern void _CbarDestroyDialUnit(t_dialunit *du)
{
   t_unit *cbcu;

   cbcu = du->cbcu;
   if (cbcu) {
      if (du->bcu && cbcu->setting.source == CBAR_READER_NONE) {
         _CbarRemoveUnit(du->bcu);
         _CbarDestroyUnit(du->bcu);
      }
      _CbarDestroyUnit(cbcu);
   }
   _CbarDestroyDialSound(du->okwr);
   _CbarDestroyDialSound(du->errorwr);
   Release(du);
}

/* The dialogue
   ------------*/

static void FlipComAct(void *data)
{
   t_dialunit *du = data;
   t_unit *cbcu;

   cbcu = du->cbcu;
   if (cbcu->setting.source >= CBAR_READER_COM1 && cbcu->setting.source <= CBAR_READER_COM4)
      Activate(du->portid);
   else
      DeActivate(du->portid);
   Refresh(du->portid);
}

static void EdComPort(void *data)
{
   t_dialunit *du = data;

   CcomSettingsDialogue(du->cbcu->setting.source - 1);
}

static void SwitchSpeaker(void *data)
{
   t_dialunit *drs = data;

   _CbarReactivateObjects(drs->okwr);
   _CbarReactivateObjects(drs->errorwr);
}

extern void _CbarMakeUnitWidgets(t_dialunit *du)
{
   t_unit *cbcu;
   int id;
   const char*const*txt;
   int nr;
   txt = LoadCompiledTexts(cbar_labels, "unit", &nr);
   if (nr < SIZE_unit)
      return;

   cbcu = du->cbcu;
   AutoHotKeys(0);
   if (du->dsource) {
      id = AddDropDownS(TOPLEFT, 0, txt[CONNECTION], &cbcu->setting.source,
                       txt + CONNECT_SELECT, 6);
      AddHandler(id, FlipComAct, du);
      du->portid = AddButton(RIGHT, txt[OPTIONS_COM], EdComPort, du);
      if (cbcu->setting.source < CBAR_READER_COM1 || cbcu->setting.source > CBAR_READER_COM4)
         DeActivate(du->portid);
   }
   if (du->dsound) {
      AddTextBox(DOWNLEFT, txt[SELECT_FREQ_DURATION_OR_WAV_FILE], 300, 0, 0);
      StartContainer(DOWNLEFT | FILLSPACE | HORIZONTAL, ADAPTIVE,
                     txt[SPEAKER], CT_BORDER);
      id = AddCheck(DOWNLEFT, txt[INTERNAL], &cbcu->setting.intsound);
      AddHandler(id, SwitchSpeaker, du);
      id = AddCheck(DOWNLEFT, txt[LEFT_SPEAKER], &cbcu->setting.leftsound);
      AddHandler(id, SwitchSpeaker, du);
      id = AddCheck(DOWNLEFT, txt[RIGHT_SPEAKER], &cbcu->setting.rightsound);
      AddHandler(id, SwitchSpeaker, du);
      EndContainer();
      _CbarMkSoundContainer(txt[CORRECT_READING], cbcu->ok, du->okwr);
      _CbarMkSoundContainer(txt[ERRONEOUS_READING], cbcu->error, du->errorwr);
   }
   AutoHotKeys(1);
}
